#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ti/csl/csl_edma3.h>
#include <ti/csl/csl_edma3Aux.h>

#pragma DATA_SECTION(srcbuf,".INRAM")
#pragma DATA_SECTION(dstbuf,".INRAM")

unsigned char srcbuf[4096],dstbuf[4096];

CSL_Edma3ParamHandle	hParamPing;
CSL_Edma3ParamSetup		myParamSetup;
CSL_Edma3ChannelHandle	hChannel;
CSL_Edma3Handle			hMoudle;

int Edma3Example(int acnt, int bcnt, int ccnt)
{
	CSL_Edma3Obj	edmaObj;
	CSL_Edma3ChannelObj		chObj;
	CSL_Edma3CmdIntr	regionIntr;
	CSL_Edma3Context	context;
	CSL_Edma3ChannelAttr	chAttr;
	CSL_Status	status;

	unsigned char channelNum = 0;
	unsigned char instNum = 1;

	CSL_edma3Init(&context);

	hMoudle = CSL_edma3Open(&edmaObj,instNum,NULL,&status);

	chAttr.regionNum = CSL_EDMA3_REGION_GLOBAL;
	chAttr.chaNum = channelNum;
	hChannel = CSL_edma3ChannelOpen(&chObj,instNum,&chAttr,&status);

	CSL_edma3HwChannelSetupQue(hChannel,CSL_EDMA3_QUE_0);

	CSL_edma3MapDMAChannelToParamBlock(hMoudle,channelNum,0);

	hParamPing = CSL_edma3GetParamHandle(hChannel,0,&status);

	regionIntr.region = CSL_EDMA3_REGION_GLOBAL;
	regionIntr.intr = 0x1;
	regionIntr.intrh = 0x0;
	CSL_edma3HwControl(hMoudle,CSL_EDMA3_CMD_INTR_ENABLE,&regionIntr);

	myParamSetup.option = 0x00900004;
	myParamSetup.srcAddr = (Uint32)srcbuf;
	myParamSetup.aCntbCnt = CSL_EDMA3_CNT_MAKE(acnt, bcnt);
	myParamSetup.dstAddr = (Uint32)dstbuf;
	myParamSetup.srcDstBidx = CSL_EDMA3_BIDX_MAKE(acnt, acnt*ccnt);
	myParamSetup.linkBcntrld = 0x0000ffff;
	myParamSetup.srcDstCidx = CSL_EDMA3_CIDX_MAKE(acnt*bcnt, acnt);
	myParamSetup.cCnt = ccnt;
	CSL_edma3ParamSetup(hParamPing,&myParamSetup);

	CSL_edma3HwChannelControl(hChannel,CSL_EDMA3_CMD_CHANNEL_SET,NULL);

	regionIntr.region = CSL_EDMA3_REGION_GLOBAL;
	regionIntr.intr = 0x0;
	regionIntr.intrh = 0x0;
	do
	{
		CSL_edma3GetHwStatus(hMoudle,CSL_EDMA3_QUERY_INTRPEND,&regionIntr);
	}
	while (!(regionIntr.intr & 0x1));

	CSL_edma3HwControl(hMoudle,CSL_EDMA3_CMD_INTRPEND_CLEAR,&regionIntr);

	CSL_edma3ChannelClose(hChannel);

	CSL_edma3Close(hMoudle);

	return 0;
}

int verify(unsigned char* src, unsigned char* dst, int acnt, int bcnt, int ccnt)
{
	int i;
	int j;
	int k;

	for(k=0; k<ccnt;k++)
	{
		for (j=0;j<bcnt;j++)
		{
			for (i=0; i<acnt; i++)
			{
				if (*(src+(k*bcnt*acnt)+(j*acnt)+i) != *(dst+(j*ccnt*acnt)+(k*acnt)+i))
				{
					return 0;
				}
			}
		}
	}

	return 1;
}

void main()
{
	int i;
	int j;
	int k;

	int acnt = 8;
	int bcnt = 4;
	int ccnt = 5;

	for(i=0;i<4096;i++)
	{
		srcbuf[i] = i;
		dstbuf[i] = 0x00;
	}

	for (i=1;i<=16;i++)
	{
		acnt = i;
		for (j=1;j<=16;j++)
		{
			bcnt =j;
			for (k=2;k<=16;k++)
			{
				ccnt = k;
				Edma3Example(acnt, bcnt, ccnt);

				if (verify(srcbuf, dstbuf, acnt, bcnt, ccnt) == 1)
				{
					printf("Test of ACNT = %2d, BCNT=%2d, CCNT = %2d: SUCC\n", acnt, bcnt, ccnt);
				}
				else
				{
					printf("Test of ACNT = %2d, BCNT=%2d, CCNT = %2d: FAIL\n", acnt, bcnt, ccnt);
				}
			}
		}
	}
}
